unit Billiard;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Menus;

type
  TSpielfeld = class(TForm)
    Uhr: TTimer;
    Bild: TImage;
    MainMenu: TMainMenu;
    Spiel: TMenuItem;
    Beginn: TMenuItem;
    N1: TMenuItem;
    Ende: TMenuItem;
    Options: TMenuItem;
    Hilfe: TMenuItem;
    Tips: TMenuItem;
    N2: TMenuItem;
    Info: TMenuItem;
    Abbruch: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure UhrTimer(Sender: TObject);
    procedure BeginnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AbbruchClick(Sender: TObject);
    procedure EndeClick(Sender: TObject);
    procedure OptionsClick(Sender: TObject);
    procedure InfoClick(Sender: TObject);
    procedure TipsClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

{ eigene Definitionen }

const FMax=15;
      Farben:Array[0..FMax]of TColor=
                     (clBlack, clMaroon, clGreen, clOlive, clNavy, clPurple,
                      clTeal, clGray, clSilver, clRed, clLime, clYellow,
                      clBlue, clFuchsia, clAqua, clWhite);

type TBall=class
  MPunkt,Delta:TPoint;
  Radius:Integer;
  Farbe:TColor;
  procedure Init(fNeu:TColor;MitteNeu,DeltaNeu:TPoint;RadiusNeu:Integer);
  procedure ZeigeDich;
  procedure BewegeDich;
  function Trifft(anderenBall:TBall):Boolean;
  function Erlaubt(K:Integer):Boolean;
  procedure DeltaTausch(anderenBall:TBall);
  end;
var
  Spielfeld:TSpielfeld;
  BallListe:Array of TBall;
  BallMax,BallRadius,N:Integer;
  KugelTonAn,BandeTonAn:Boolean;

implementation

uses Einstellungen, Info, Regeln;
{$R *.dfm}
{$R+,Q+}

 procedure TBall.Init(fNeu:TColor;MitteNeu,DeltaNeu:TPoint;RadiusNeu:Integer);
  begin
   Farbe:=fneu;
   Radius:=RadiusNeu;
   MPunkt:=MitteNeu;
   Delta:=DeltaNeu;
  end;

 procedure TBall.ZeigeDich;
  begin
   with Spielfeld.Bild.Canvas do
    begin
     Brush.Color:=Farbe;
     Ellipse(MPunkt.X-Radius,MPunkt.Y-Radius,MPunkt.X+Radius,MPunkt.Y+Radius)
    end;
  end;

 procedure TBall.BewegeDich;
  begin
   MPunkt.X:=MPunkt.X+Delta.X;
   MPunkt.Y:=MPunkt.Y+Delta.Y;
   with Spielfeld.Bild do
    begin
     if (MPunkt.X>Width-Radius-1) then
      begin
       if BandeTonAn then Beep;
       MPunkt.X:=Width-Radius-1;
       Delta.X:=-Delta.X;
      end;
     if MPunkt.X<Radius+1 then
      begin
       if BandeTonAn then Beep;
       MPunkt.X:=Radius+1;
       Delta.X:=-Delta.X;
      end;
     if (MPunkt.Y>Height-Radius-1) then
      begin
       if BandeTonAn then Beep;
       MPunkt.Y:=Height-Radius-1;
       Delta.Y:=-Delta.Y;
      end;
     if MPunkt.Y<Radius+1 then
      begin
       if BandeTonAn then Beep;
       MPunkt.Y:=Radius+1;
       Delta.Y:=-Delta.Y;
      end
    end
  end;

 function TBall.trifft(anderenBall:TBall):Boolean;
  begin
   Result:=False;
   if sqr(MPunkt.X-anderenBall.MPunkt.X)+
      sqr(MPunkt.Y-anderenBall.MPunkt.Y)<=
      sqr(Radius+anderenBall.Radius) then Result:=True
  end;

 procedure TBall.DeltaTausch(anderenBall:TBall);
  var Tausch:TPoint;
  begin
   Tausch:=Delta;
   Delta:=anderenBall.Delta;
   anderenBall.Delta:=Tausch;
  end;

 function TBall.Erlaubt(K:Integer):Boolean;
  var I:Integer;
  begin
   Result:=True;
   for I:=0 to K-1 do
    if sqr(BallListe[I].MPunkt.X-BallListe[K].MPunkt.X)+
       sqr(BallListe[I].MPunkt.Y-BallListe[K].MPunkt.Y)<
       sqr(BallListe[I].Radius+BallListe[K].Radius) then
        begin Result:=False;exit end
  end;

 procedure TSpielfeld.UhrTimer(Sender: TObject);
  var K,L:Integer;
  begin
   for K:=0 to BallMax do
    begin
     BallListe[K].ZeigeDich;
     BallListe[K].BewegeDich;
     BallListe[K].ZeigeDich;
    end;
   for K:=0 to BallMax-1 do
    for L:=K+1 to BallMax do
     if BallListe[K].trifft(BallListe[L]) then
      begin
       if KugelTonAn then Beep;
       BallListe[K].DeltaTausch(BallListe[L]);
      end;
  end;

 procedure TSpielfeld.BeginnClick(Sender: TObject);
  const VMax=10000;
  var M,R:TPoint;
      K,Versuch:Integer;
  begin
   BallRadius:=Optionen.SpinEditBR.Value;
   BallMax:=Optionen.SpinEditBA.Value;
   KugelTonAn:=Optionen.KugelSound.Checked;
   BandeTonAn:=Optionen.BandeSound.Checked;
   with Bild.Canvas do
    begin
     Pen.Mode:=pmCopy;
     Brush.Color:=clWhite;
     RecTangle(0,0,Bild.width,Bild.Height);
     Pen.Width:=2;
     Brush.Style:=bsSolid;
     Pen.Mode:=pmNotXor;
    end;
   SetLength(BallListe,BallMax+1);
   for K:=0 to BallMax do BallListe[K]:=TBall.Create;
   M.X:=random(Bild.Width-50)+25;
   M.Y:=random(Bild.Height-50)+25;
   R.X:=random(9)-4;
   R.Y:=random(9)-4;
   BallListe[0].Init(Farben[0],M,R,BallRadius);
   BallListe[0].ZeigeDich;
   for K:=1 to BallMax do
    begin
     Versuch:=0;
     repeat
      inc(Versuch);
      M.X:=random(Bild.Width-50)+25;
      M.Y:=random(Bild.Height-50)+25;
      R.X:=random(9)-4;
      R.Y:=random(9)-4;
      if K<=FMax then BallListe[K].Init(Farben[K],M,R,BallRadius)
      else BallListe[K].Init(Farben[random(FMax+1)],M,R,BallRadius);
     until BallListe[K].Erlaubt(K) or (Versuch=VMax);
     BallListe[K].ZeigeDich;
    end;
   Uhr.Enabled:=True; 
  end;

 procedure TSpielfeld.FormCreate(Sender: TObject);
  var K:Integer;
  begin
   Randomize;BallMax:=4;BallRadius:=20;
   Uhr.Enabled:=False;
   SetLength(BallListe,BallMax+1);
   for K:=0 to BallMax do BallListe[K]:=TBall.Create;
   with Bild.Canvas do
    begin
     Pen.Width:=5;
     Brush.Color:=clWhite;
     RecTangle(0,0,Bild.width,Bild.Height);
     Pen.Width:=2;
     Brush.Style:=bsSolid;
     Pen.Mode:=pmNotXor;
    end;
  end;

 procedure TSpielfeld.FormDestroy(Sender: TObject);
  var N:Integer;
  begin
    for N:=0 to BallMax do BallListe[N].Destroy;
  end;

 procedure TSpielfeld.AbbruchClick(Sender: TObject);
  begin Uhr.Enabled:=False end;

 procedure TSpielfeld.EndeClick(Sender: TObject);
  begin close end;

 procedure TSpielfeld.OptionsClick(Sender: TObject);
  begin
   Optionen.Show;
  end;

 procedure TSpielfeld.InfoClick(Sender: TObject);
  begin
   InfoBox.ShowModal;
  end;

 procedure TSpielfeld.TipsClick(Sender: TObject);
  begin
   RegelnBox.ShowModal;
  end;

end.

